'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convert = convert;
exports.wrapSchema = wrapSchema;
exports.convertSchema = convertSchema;
exports.convertClassName = convertClassName;
exports.convertFieldGroups = convertFieldGroups;
exports.convertFieldsets = convertFieldsets;
exports.convertFields = convertFields;
exports.convertObjectArray = convertObjectArray;

var _promise = require('promise');

var _promise2 = _interopRequireDefault(_promise);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _validate = require('./validate');

var _renderer = require('./renderer');

var _transforms = require('./transforms');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function convert(uis1, outfile, detail, logger) {
  return convertSchema(uis1, logger, detail).then(function (uis2) {
    logger.log(uis2);
    return wrapSchema(uis2, detail, logger);
  }).then(function (uis2) {
    logger.log(JSON.stringify(uis2, null, 2));
    logger.log('attempting to validate view');
    return (0, _validate.validateView)(uis2, logger).then(function (result) {
      return result[0];
    });
  });
}

function wrapSchema(uis2, detail) {
  return _promise2.default.resolve({
    type: detail ? 'detail' : 'form',
    version: '2.0',
    cells: uis2.children
  });
}

function convertSchema(ui1, logger, detail) {
  var newObj = {};
  logger.log('converting...');
  return convertClassName(ui1, newObj, logger).then(function (ui2) {
    var key = _lodash2.default.keys(ui1)[0];
    if (ui1[key].fieldGroups) {
      return convertFieldGroups(ui1, ui2, logger, detail);
    }
    if (ui1[key].fields) {
      logger.log('found fields instead of fieldgroups');
      ui2.children = convertFields(ui1[key].fields, logger, detail);
      return _promise2.default.resolve(ui2);
    }
  }).then(function (uis2) {
    uis2.children.splice(0, 0, {
      children: [{ model: 'label' }, { model: 'description' }],
      label: 'General'
    });

    return uis2;
  });
}

function convertClassName(ui1, ui2, logger) {
  logger.log('converting class name');
  var key = _lodash2.default.keys(ui1)[0];
  ui2.classNames = {
    cell: ui1[key].cssClass || ''
  };
  return _promise2.default.resolve(ui2);
}

function convertFieldGroups(ui1, ui2, logger, detail) {
  logger.log('converting field groups');
  var key = _lodash2.default.keys(ui1)[0];
  logger.log(ui1);
  var fgs = ui1[key].fieldGroups || [];
  var children = fgs.map(function (fg) {
    var fields = convertFields(fg.fields, logger, detail).concat(convertFieldsets(fg.fieldsets, logger));
    return {
      'label': fg.name || '',
      'children': fields
    };
  });
  ui2.children = children;
  return _promise2.default.resolve(ui2);
}

/*eslint-disable complexity */
function convertFieldsets(fieldsets, logger, detail) {
  logger.log('converting fieldsets');
  return _lodash2.default.map(fieldsets, function (fieldset, key) {
    logger.log('key: ' + key);
    var newFieldset = {
      model: 'properties.' + key.split('_').join(''),
      collapsible: !!fieldset['switch'] || true,
      children: convertFields(fieldset.fields, logger, detail)
    };
    if (fieldset.label) {
      newFieldset.label = fieldset.label;
    }
    if (fieldset.help || fieldset.description) {
      newFieldset.description = fieldset.help || fieldset.description;
    }
    return newFieldset;
  });
}

function convertFields(fields, logger, detail) {
  logger.log('converting fields...');
  return _lodash2.default.map(fields, function (field, key) {
    var newField = {
      model: 'properties.' + key
    };
    if (field.type === 'objectarray') {
      newField.arrayOptions = convertObjectArray(field, logger);
    }
    if (field.label) {
      newField.label = field.label;
    }
    if (field.help || field.description) {
      newField.description = field.help || field.description;
    }
    var placeholder = field.placeholder || field.prompt;
    if (placeholder) _lodash2.default.extend(newField, { placeholder: placeholder });
    (0, _transforms.setTransforms)(newField, field, logger);

    return (0, _renderer.setRenderer)(newField, field, logger, detail);
  });
}

/* eslint-enable complexity */
function convertObjectArray(field, logger) {
  var result = {
    autoAdd: true,
    compact: true,
    showLabel: false,
    sortable: true
  };
  if (field.order) {
    result.itemCell = {
      children: _lodash2.default.map(field.order.split(','), function (prop) {
        return { model: 'properties.' + prop };
      })
    };
  }
  return result;
}