'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertInstructions = convertInstructions;
exports.getInstructorValue = getInstructorValue;
exports.setModelType = setModelType;
exports.setQuery = setQuery;
exports.smartSet = smartSet;
exports.setRenderer = setRenderer;
exports.setRendererOptions = setRendererOptions;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _i = require('i');

var _i2 = _interopRequireDefault(_i);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var inflect = (0, _i2.default)();

function convertInstructions(field, logger) {
  if (!field.instructions || !field.instructions.length) {
    return;
  }
  var operators = {
    'contains': {
      values: [],
      param: 'p'
    },
    'equals': {
      param: 'q',
      values: []
    },
    'byProvider': {
      param: 'domainId',
      template: 'value',
      values: []
    }
  };
  _lodash2.default.each(field.instructions, function (inst, key) {
    inst = inst.and || inst.or || inst;
    if (inst.type === 'filter') {
      if (operators[inst.operator] && operators[inst.operator].param) {
        operators[inst.operator].values.push(getInstructorValue(inst, field));
      } else {
        logger.warn('The instruction operator ' + inst.operator + ' is not supported at this time.');
      }
    }
  });
  return _lodash2.default.reduce(operators, function (result, operator, key) {
    if (operator.values.length) {
      result[operator.param] = operator.values.join(',');
    }
    return result;
  }, {});
}

function getInstructorValue(inst, field) {
  var value = '${' + inst.value + '}';
  if (inst.value && inst.value.split && inst.value.split('\'').length === 3) {
    value = inst.value.split('\'')[1];
  }
  if (inst.value === 'instructor') {
    value = '${./' + field.instructor + '}';
  }
  if (inst.key) {
    value = inst.key + ':' + value;
  }
  return value;
}

function setModelType(options, field) {
  if (field.resourceType) {
    if (field.resourceType.split('.').length === 3) {
      options.modelType = 'resource';
      return;
    }
    options.modelType = inflect.singularize(field.resourceType);
  }
}

function setQuery(options, field, logger) {
  var query = {};
  if (field.resourceType && field.resourceType.split('.').length === 3) {
    _lodash2.default.extend(query, { resourceTypeId: field.resourceType });
  }
  if (field.instructions) {
    _lodash2.default.extend(query, convertInstructions(field, logger));
  }
  if (!_lodash2.default.isEmpty(query)) {
    options.query = query;
  }
}

function smartSet(options, rAttr, uisField, fAttr) {
  fAttr = fAttr || rAttr;
  if (uisField[fAttr]) {
    options[rAttr] = uisField[fAttr];
  }
}

function setRenderer(newField, field, logger, detail) {
  if (!field.type || detail) {
    return newField;
  }
  var renderers = {
    'select': {
      name: 'select'
    }
  };
  var renderer = renderers[field.type];
  if (renderer) {
    setRendererOptions(renderer, field, logger);
    newField.renderer = renderer;
  }
  return newField;
}

function setRendererOptions(renderer, uis1Field, logger) {
  var options = {};
  setModelType(options, uis1Field);
  smartSet(options, 'valueAttribute', uis1Field);
  smartSet(options, 'labelAttribute', uis1Field);
  setQuery(options, uis1Field, logger);
  if (!_lodash2.default.isEmpty(options)) {
    renderer.options = options;
  }
}